@@ -64,7 +64,7 @@ module Data.Constraint
   , strengthen1, strengthen2
   , (&&&), (***)
   , trans, refl
-  , Bottom
+  , Bottom(no)
   , top, bottom
   -- * Dict is fully faithful
   , mapDict
@@ -75,12 +75,11 @@ module Data.Constraint
   ) where
 import Control.Applicative
 import Control.Category
+import Control.DeepSeq
 import Control.Monad
-#if __GLASGOW_HASKELL__ < 710
-import Data.Monoid
-#endif
 import Data.Complex
 import Data.Ratio
+import Data.Semigroup
 import Data.Data
 import qualified GHC.Exts as Exts (Any)
 import GHC.Exts (Constraint)
@@ -132,6 +131,9 @@ deriving instance Eq (Dict a)
 deriving instance Ord (Dict a)
 deriving instance Show (Dict a)
 
+instance NFData (Dict c) where
+  rnf Dict = ()
+
 -- | From a 'Dict', takes a value in an environment where the instance
 -- witnessed by the 'Dict' is in scope, and evaluates it.
 --
@@ -238,6 +240,9 @@ instance Ord (a :- b) where
 instance Show (a :- b) where
   showsPrec d _ = showParen (d > 10) $ showString "Sub Dict"
 
+instance a => NFData (a :- b) where
+  rnf (Sub Dict) = ()
+
 infixl 1 \\ -- required comment
 
 -- | Given that @a :- b@, derive something that needs a context @b@, using the context @a@
@@ -450,6 +455,7 @@ instance () :=> Show () where ins = Sub Dict
 instance () :=> Show Bool where ins = Sub Dict
 instance () :=> Show Ordering where ins = Sub Dict
 instance () :=> Show Char where ins = Sub Dict
+instance () :=> Show Int where ins = Sub Dict
 instance Show a :=> Show (Complex a) where ins = Sub Dict
 instance Show a :=> Show [a] where ins = Sub Dict
 instance Show a :=> Show (Maybe a) where ins = Sub Dict
@@ -471,6 +477,7 @@ instance () :=> Read () where ins = Sub Dict
 instance () :=> Read Bool where ins = Sub Dict
 instance () :=> Read Ordering where ins = Sub Dict
 instance () :=> Read Char where ins = Sub Dict
+instance () :=> Read Int where ins = Sub Dict
 instance Read a :=> Read (Complex a) where ins = Sub Dict
 instance Read a :=> Read [a] where ins = Sub Dict
 instance Read a :=> Read (Maybe a) where ins = Sub Dict
@@ -618,8 +625,27 @@ instance RealFloat a :=> RealFloat (Identity a) where ins = Sub Dict
 instance RealFloat a :=> RealFloat (Const a b) where ins = Sub Dict
 #endif
 
+-- Semigroup
+instance Class () (Semigroup a) where cls = Sub Dict
+instance () :=> Semigroup () where ins = Sub Dict
+instance () :=> Semigroup Ordering where ins = Sub Dict
+instance () :=> Semigroup [a] where ins = Sub Dict
+instance Semigroup a :=> Semigroup (Maybe a) where ins = Sub Dict
+instance (Semigroup a, Semigroup b) :=> Semigroup (a, b) where ins = Sub Dict
+instance Semigroup a :=> Semigroup (Const a b) where ins = Sub Dict
+#if MIN_VERSION_base(4,9,0)
+instance Semigroup a :=> Semigroup (Identity a) where ins = Sub Dict
+#endif
+#if MIN_VERSION_base(4,10,0)
+instance Semigroup a :=> Semigroup (IO a) where ins = Sub Dict
+#endif
+
 -- Monoid
+#if MIN_VERSION_base(4,11,0)
+instance Class (Semigroup a) (Monoid a) where cls = Sub Dict
+#else
 instance Class () (Monoid a) where cls = Sub Dict
+#endif
 instance () :=> Monoid () where ins = Sub Dict
 instance () :=> Monoid Ordering where ins = Sub Dict
 instance () :=> Monoid [a] where ins = Sub Dict
@@ -628,8 +654,6 @@ instance (Monoid a, Monoid b) :=> Monoid (a, b) where ins = Sub Dict
 instance Monoid a :=> Monoid (Const a b) where ins = Sub Dict
 #if MIN_VERSION_base(4,9,0)
 instance Monoid a :=> Monoid (Identity a) where ins = Sub Dict
-#endif
-#if MIN_VERSION_base(4,9,0)
 instance Monoid a :=> Monoid (IO a) where ins = Sub Dict
 #endif
 
@@ -700,7 +724,13 @@ instance a => Bounded (Dict a) where
 instance a :=> Read (Dict a) where ins = Sub Dict
 deriving instance a => Read (Dict a)
 
+instance () :=> Semigroup (Dict a) where ins = Sub Dict
+instance Semigroup (Dict a) where
+  Dict <> Dict = Dict
+
 instance a :=> Monoid (Dict a) where ins = Sub Dict
 instance a => Monoid (Dict a) where
-  mappend Dict Dict = Dict
+#if !(MIN_VERSION_base(4,11,0))
+  mappend = (<>)
+#endif
   mempty = Dict
